//
// $Id$
//
// Author:  Jens-S. Vckler <voeckler@rvs.uni-hannover.de>
//
// File:    vector.hh
//          Thu Aug 05 1999
//
// (c) 1998 Lehrgebiet Rechnernetze und Verteilte Systeme
//          Universitt Hannover, Germany
//
// Permission to use, copy, modify, distribute, and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that (i) the above copyright notices and this permission
// notice appear in all copies of the software and related documentation,
// and (ii) the names of the Lehrgebiet Rechnernetze und Verteilte
// Systeme and the University of Hannover may not be used in any
// advertising or publicity relating to the software without the
// specific, prior written permission of Lehrgebiet Rechnernetze und
// Verteilte Systeme and the University of Hannover.
//
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
//
// IN NO EVENT SHALL THE LEHRGEBIET RECHNERNETZE UND VERTEILTE SYSTEME OR
// THE UNIVERSITY OF HANNOVER BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
// INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT
// ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY,
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
// SOFTWARE.
//
// $Log$
//
#ifndef _VECTOR_HH
#define _VECTOR_HH

#if defined(__GNUC__) || defined(__GNUG__)
#pragma interface
#endif

#include "typedefs.h"
#include "counter.hh"

template <class T> class Vector {
  // growable start-with-0 indexed vector. 
  // for a different start use OffsetVector.
public:
  inline Vector()
    :array(0),_size(0) { }
  inline ~Vector()
    { delete[] array; array = 0; }
  inline Vector( const Vector& gv )
    :array( new T[gv._size] ),_size(gv._size)
    { for ( int i=0; i<_size; i++ ) array[i] = gv.array[i]; }

  inline size_t size() const { return _size; }
  
  Vector& operator=( const Vector& gv )
    {
      if ( &gv != this ) {
	delete[] array;
	_size = gv._size;
	for ( int i=0; i<_size; i++ ) array[i] = gv.array[i];
      }
      return *this;
    }

  T& operator[]( MyUInt32 index )
    {
      if ( array == 0 ) {
	// first growth
	array = new T[ (_size=index+1) ];
      } else if ( index >= _size ) {
	// positive growth necessary
	size_t newsize = index+1;
	T* newarray = new T[newsize];
	for ( size_t i=0; i<_size; ++i ) newarray[i] = array[i];
	delete[] array;
	_size = newsize;
	array = newarray;
      }
      return array[index];
    }

  inline T operator[]( MyUInt32 index ) const
    {
      if ( array == 0 || index >= _size ) return T();
      else return array[index];
    }

protected:
  T*        array;
  size_t    _size;
};

#endif // _GROW_HH
