%{
/* 
 * $Id: config.l,v 1.9 2000/07/27 07:38:22 voeckler Exp $
 *
 * Author:  Jens-S. Vckler <voeckler@rvs.uni-hannover.de>
 *
 * File:    config.l
 *          Thu Aug 05 1999
 *
 * (c) 1999 Lehrgebiet Rechnernetze und Verteilte Systeme
 *          Universitt Hannover, Germany
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notices and this permission
 * notice appear in all copies of the software and related documentation,
 * and (ii) the names of the Lehrgebiet Rechnernetze und Verteilte
 * Systeme and the University of Hannover may not be used in any
 * advertising or publicity relating to the software without the
 * specific, prior written permission of Lehrgebiet Rechnernetze und
 * Verteilte Systeme and the University of Hannover.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL THE LEHRGEBIET RECHNERNETZE UND VERTEILTE SYSTEME OR
 * THE UNIVERSITY OF HANNOVER BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT
 * ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 * $Log: config.l,v $
 * Revision 1.9  2000/07/27 07:38:22  voeckler
 * created new configuration tags for suffix lists, suffix list implementation,
 * internal clients, internal client list length, extendend internal list and
 * configurable internal path component prefix.
 *
 * Revision 1.8  1999/11/29 15:04:17  voeckler
 * added separate flag to warn about status codes which are neither TCP nor UDP,
 * but which are counted as TCP. Formerly, the unknown status flag was also
 * responsible for this.
 *
 * Revision 1.7  1999/09/02 10:12:59  voeckler
 * added external helper processes (exec file name and nr of helpers) config.
 *
 * Revision 1.6  1999/08/26 19:41:35  voeckler
 * added dnsChildren configuration parameter.
 *
 * Revision 1.5  1999/08/25 21:20:37  voeckler
 * added new configuration options to the grammar, e.g. distribution
 * and user selectable table lengths.
 *
 * Revision 1.4  1999/08/23 12:42:37  voeckler
 * added daily interval to config file parser.
 *
 * Revision 1.3  1999/08/22 11:13:32  voeckler
 * added duration vs. data rate option.
 *
 * Revision 1.2  1999/08/05 22:08:07  voeckler
 * removed references to unused data structure symtrie.
 *
 * Revision 1.1  1999/08/05 21:01:20  voeckler
 * Initial revision
 *
 *
 */
#include <string.h>
#include <stdlib.h>
#define YY_SKIP_YYWRAP true

#include "typedefs.h"
#include "global.hh"
#include "symtrie32.hh"
#include "symtrie128.hh"
#include "lltrie.hh"
#include "config.h"

extern MyUInt32 lineno;
extern int columns;
extern char parserErrorSeen;

int yywrap() 
{ 
  // some bison action code
  if ( columns ) fputc( '\n', stderr );
  return 1; 
}

#define MAXBUFF 1023
char linebuf[MAXBUFF+1];
%}

qstring \"[^\n\"]*\"
id      [A-Za-z_][A-Za-z0-9_]*
lelem	[A-Z][A-Z_]+

xdigit	[0-9A-Fa-f]
digit	[0-9]

%%
#[^\n]*		; /* do nothing (a comment) */

end		{ /* explicit end of file: flex yyterminate(); */
		  return 0;
		}

[Tt][Rr][Uu][Ee]	{
			  yylval.boolean = true;
			  return BOOLEAN;
			}
[Ff][Aa][Ll][Ss][Ee]	{
			  yylval.boolean = false;
			  return BOOLEAN;
			}

debug_level	{ return CFG_DEBUG; }
no_ident	{ return CFG_IDENT; }
allowed_whitespaces { return CFG_NR_WHITE; }
prefer_datarate { return CFG_DATARATE; }
peak_interval	{ return CFG_IV_PEAK; }
daily_interval  { return CFG_IV_DAILY; }

log_fqdn	{ return CFG_LOG_FQDN; }
dns_cache_file	{ return CFG_DNS_FILE; }
dns_positive_ttl { return CFG_DNS_POS_TTL; }
dns_negative_ttl { return CFG_DNS_NEG_TTL; }
dns_length_tlds	{ yylval.numeric = Global::S_TLD; return TABLE_SIZE; }
dns_length_slds	{ yylval.numeric = Global::S_2LD; return TABLE_SIZE; }
dns_children	{ return CFG_DNS_CHILDREN; }
dns_program	{ return CFG_DNS_HELPER; }

irr_cache_file	{ return CFG_IRR_FILE; }
irr_server	{ return CFG_IRR_SERVER; }
irr_positive_ttl { return CFG_IRR_POS_TTL; }
irr_negative_ttl { return CFG_IRR_NEG_TTL; }
irr_length_asns	{ yylval.numeric = Global::S_ASN; return TABLE_SIZE; }
irr_children	{ return CFG_IRR_CHILDREN; }
irr_program	{ return CFG_IRR_HELPER; }

client_length_udp	{ yylval.numeric = Global::S_UDP; return TABLE_SIZE; }
client_length_tcp	{ yylval.numeric = Global::S_TCP; return TABLE_SIZE; }
client_length_internal  { yylval.numeric = Global::S_INT; return TABLE_SIZE; }

warn_crash_interval	{ return CFG_WARN_CRASH; }
show_distributions	{ return CFG_SHOW_DIST; }
extended_internal_clients { return CFG_EXT_ICLIENT; }
internal_urlpath_prefix { return CFG_IURL_PREFIX; }

warn_unknown_method	{ return CFG_WARN_METHOD; }
warn_unknown_scheme	{ return CFG_WARN_SCHEME; }
warn_unknown_suffix	{ return CFG_WARN_SUFFIX; }
warn_unknown_hierarchy	{ return CFG_WARN_HIER; }
warn_unknown_status	{ return CFG_WARN_STATUS; }
warn_none_as_tcp	{ return CFG_WARN_NONE_TCP; }
warn_unknown_tld	{ return CFG_WARN_TLD; }

method_list	{ return CFG_METHOD; }
hierarchy_list	{ return CFG_HIER; }
status_list	{ return CFG_STATUS; }
mediatype_list	{ return CFG_MEDIA; }
media_subtypes	{ return CFG_SUBTYPES; }
scheme_list	{ return CFG_SCHEME; }
domain_list     { return CFG_DOMAIN; }
suffix_list	{ return CFG_SUFFIX; }
alias		{ return OPT_ALIAS; }

tcp		{ return OPT_TCP; }
udp		{ return OPT_UDP; }
hit		{ return OPT_HIT; }
miss		{ return OPT_MISS; }
none		{ return OPT_NONE; }
direct		{ return OPT_DIRECT; }
peer		{ return OPT_PEER; }
parent		{ return OPT_PARENT; }

0[xX]{xdigit}+	{ /* numerical value: hexadecimal */
		  yylval.numeric = strtoul(yytext,0,16);
		  return INTEGER;
		}
0{digit}*	{ /* numerical value: octal */
		  yylval.numeric = strtoul(yytext,0,8);
		  return INTEGER;
		}
[1-9]{digit}*	{ /* numerical value */
		  yylval.numeric = strtoul(yytext,0,10);
		  return INTEGER;
		}

{id}		{ /* identifier */
		  yylval.lexxer = strdup(yytext);
		  return ID;
		}

{qstring}       { // found a quoted string. Quotes within *are* allowed,
		  // but must be escaped using the C backslash notation.
		  if ( yytext[yyleng-2] == '\\' ) {
		    yyless(yyleng-1);
		    yymore();
		  } else {
		    if ( strlen(yytext) > 2 ) {
                      yytext[strlen(yytext)-1]='\0';
                      yylval.lexxer = strdup(yytext+1);
                    } else {
                      yylval.lexxer = NULL; // empty string
		    }
                    return QSTRING;
		  }
                }

\n.*		{ 
		  strncpy( linebuf, yytext+1, MAXBUFF );
		  lineno++; 
		  yyless(1);
		}
[\t\r ]+	;
.               return yytext[0]; /* error or one-char-token */
%%

#include <stdio.h>

static const char *RCS_ID = 
  "$Id";

void
yyerror( const char* msg )
{
  parserErrorSeen = 1;
  fprintf( stderr, SF_U32 ": %s near \"%s\":\n%s\n", 
		   lineno, msg, yytext, linebuf );
}
