#! /sw/perl-5.005/bin/perl
require 5.005;
use strict;
use DBI;

use File::Basename;
use lib dirname($0);
use Scripts qw(@mon);
use Wrapper;

# use eval() for database connection in order to die gracefully on errors
my ($start,$final,@cache);
eval {
    &database_login;

    my $q = 'select min(first),max(first) from sf_stamp2';
    my $qh = $Apache::Registry::sf->prepare($q) or
	die "cannot prepare($q): ", $DBI::errstr;
    my $rc = $qh->execute or
	die "cannot execute($q): ", $DBI::errstr;
       ($start,$final) = $qh->fetchrow_array;
    $qh->finish;

    $q = "select distinct cache from sf_stamp1 order by cache";
    $qh = $Apache::Registry::sf->prepare($q) or
        die "cannot prepare($q): ", $Apache::Registry::sf->errstr;
    my $rc = $qh->execute or
        die "cannot execute($q): ", $Apache::Registry::sf->errstr;
    my @row;
    push @cache, $row[0] while ( @row = $qh->fetchrow_array );
    undef @row;
    $qh->finish;

    $Apache::Registry::sf->disconnect() or
	die "disconnect: $DBI::errstr";
};

# check for errors during db connection, cluck and exit
if ( $@ ne '' ) {
    print << "EOF";
Content-Type: text/plain; charset=iso-8859-1
Cache-Control: private; max-age=0

Error during execution of database connection:

$@
EOF
    exit(1);
}

# convert UTC stamps into local time
my @start = localtime($start);
my @final = localtime($final);

sub create (\@$$$) {
    my ($array,$start,$stop,$current) = @_;
    for ( my $i=$start; $i<$stop; ++$i ) {
        push @{$array}, sprintf( "<OPTION%s>%02d</OPTION>\n",
                                 $i == $current ? " SELECTED" : "", $i );
    }
}

sub create_mon (\@$$$) {
    my ($array,$start,$stop,$current) = @_;
    for ( my $i=$start; $i<$stop; ++$i ) {
	push @{$array}, sprintf( "<OPTION%s VALUE=\"%d\">%3s</OPTION>\n",
				 $i == $current ? " SELECTED" : "", $i,
				 $mon[$i] );
    }
}

my (@shour,@sday,@smon,@syear);
create( @shour, 0, 24, $start[2] );
create( @sday, 1, 32, $start[3] );
create_mon( @smon, 0, 12, $start[4] );
for ( my $i = 1997; $i < 2010; ++$i ) {
    push @syear, sprintf( "<OPTION%s>%04d</OPTION>\n",
                           $i == 1900+$start[5] ? " SELECTED" : "",
                           $i );
}

my (@fhour,@fday,@fmon,@fyear);
create( @fhour, 0, 24, $final[2] );
create( @fday, 1, 32, $final[3] );
create_mon( @fmon, 0, 12, $final[4] );
for ( my $i = 1997; $i < 2010; ++$i ) {
    push @fyear, sprintf( "<OPTION%s>%04d</OPTION>\n",
                           $i == 1900+$final[5] ? " SELECTED" : "",
                           $i );
}
@cache = map { "<OPTION>$_</OPTION>\n" } @cache;

# create input form for next stage
print << "EOF";
Content-Type: text/html; charset=iso-8859-1
Cache-Control: public; max-age=86400

<HTML>
<HEAD>
<TITLE>Selection</TITLE>
</HEAD>
<BODY BGCOLOR="white">
<H1>Selection</H1>

The smallest time stamp in the database is $start aka
@{[scalar localtime($start)]},<BR>
and the largest time stamp is $final aka
@{[scalar localtime($final)]}.

<FORM ACTION="ctor.cgi" METHOD="GET">
<TABLE BORDER="0" BGCOLOR="#CCCCFF" CELLSPACING="0" CELLPADDING="2">
<TR>
 <TH ALIGN="right">Start Time:</TH>
 <TD><SELECT NAME="shour" SIZE="1">@shour
     </SELECT>hours, </TD>
 <TD><SELECT NAME="sday" SIZE="1">@sday
     </SELECT>.</TD>
 <TD><SELECT NAME="smon" SIZE="1">@smon
     </SELECT>.</TD>
 <TD><SELECT NAME="syear" SIZE="1">@syear
     </SELECT></TD>
</TR>
<TR>&nbsp;</TR>
<TR>
 <TH ALIGN="right">Final Time:</TH>
 <TD><SELECT NAME="fhour" SIZE="1">@fhour
     </SELECT>hours, </TD>
 <TD><SELECT NAME="fday" SIZE="1">@fday
     </SELECT>.</TD>
 <TD><SELECT NAME="fmon" SIZE="1">@fmon
     </SELECT>.</TD>
 <TD><SELECT NAME="fyear" SIZE="1">@fyear
     </SELECT></TD>
</TR>
<TR>&nbsp;</TR>
<TR>
 <TH ALIGN="right">Hosts:</TH>
 <TD COLSPAN="4"><SELECT NAME="cache" SIZE="@{[0+@cache]}" MULTIPLE>@cache
     </SELECT></TD>
</TR>
<TR>&nbsp;</TR>
<TR>
 <TH ALIGN="right">Diagram:</TH>
 <TD COLSPAN="2">
  <INPUT TYPE="radio" NAME="isize" VALUE="0"> small<BR>
  <INPUT TYPE="radio" CHECKED NAME="isize" VALUE="1"> medium<BR>
  <INPUT TYPE="radio" NAME="isize" VALUE="2"> large<BR>
 </TD> 
 <TD>
   <INPUT TYPE="radio" CHECKED NAME="format" VALUE="0"> GIF</BR>
   <INPUT TYPE="radio" NAME="format" VALUE="1"> PNG</BR>
 </TD>
 <TD>
   <INPUT TYPE="radio" NAME="format" VALUE="2"> EPS</BR>
   <INPUT TYPE="radio" NAME="format" VALUE="3"> AI</BR>
 </TD>
</TR>
<TR>&nbsp;</TR>
<TR>
 <TH ALIGN="right">Query:</TH>
 <TD COLSPAN="3"><SELECT NAME="type" SIZE="1">
    <OPTION VALUE="0">Peak TCP and TCP-HIT</OPTION>
    <OPTION VALUE="1">non-GET methods</OPTION>
    <OPTION VALUE="2">*.win-ip.dfn.de TCP clients</OPTION>
    <OPTION VALUE="3" SELECTED>IN, OUT and AMONG</OPTION>
    <OPTION VALUE="4">Frequent toplevel domains</OPTION>
    <OPTION VALUE="5">Frequent MIME types</OPTION>
    <OPTION VALUE="6">Top N toplevel domains</OPTION>
    <OPTION VALUE="7">Top N AS destinations</OPTION>
    <OPTION VALUE="8">Top N UDP clients</OPTION>
    <OPTION VALUE="9">Top N UDP HIT clients</OPTION>
    <OPTION VALUE="10">Top N TCP clients</OPTION>
    <OPTION VALUE="11">Top N TCP HIT clients</OPTION>
    <OPTION VALUE="12">Hierarchy performance</OPTION>
     </SELECT></TD>
 <TD>
    <INPUT TYPE="radio" CHECKED NAME="kind" VALUE="0"> by req.</BR>
    <INPUT TYPE="radio" NAME="kind" VALUE="1"> by vol.</BR>
 </TD>
</TR>
<TR>&nbsp;</TR>
<TR>
 <TH ALIGN="right">Action:</TH>
 <TD COLSPAN="2"><INPUT TYPE="submit" VALUE="submit"></TD>
 <TD COLSPAN="2"><INPUT TYPE="reset" VALUE="reset"></TD>
</TR>
</TABLE>
</FORM>
<P>

<H1>Some Explanations</H1>

Please check your diagram timestamp very carefully, and use shift-reload to
obtain the correct diagram!

<H3>Peak TCP and TCP-HIT</H3>

The peak performance is a simple query into the perfomance table of the
database. Please note that for every host, two graphs will be displayed.

<H3>non-GET methods</H3>

The HTTP method GET is major part of all possible methods. This slightly
more complex query displays two graphs for each selected host, one with the
! (GET or ICP_QUERY) queries, and one with the HITs in those. An improved
version might show the percentage of !GET methods with regards to all
methods.

<H3>*.win-ip.dfn.de TCP clients</H3>

Hosts in the domain win-ip.dfn.de constitute the correct reverse resolution
of our top level caches. For each selected cache host, the graph will show
the sum of other nine caches querying the selected cache.

<H3>IN, OUT and AMONG</H3>

The basic data to evaluate the efficiency of a cache are of two kinds: the
savings OUT minus IN, and the overhead AMONG. For the given diagram, the
selected hosts are put into one query group, for which the amount IN, OUT
and AMONG is determined. Please note that the AMONG traffic is - in our
case completely - limited to the backbone. Also note the new section CD,
which is a drop-in for all squid-internal traffic like cache digests, 
network databases and ftp icons.

<BLOCKQUOTE><IMG SRC="balance.gif" ALT="balance"></BLOCKQUOTE><P>

The information for IN is determined from all hierarchy types DIRECT, the
value for AMONG from the sum of hierarchy types PEER and PARENT, and the
information for OUT is determined from the TCP client list minus any DFN
cache traffic.<P>

Of course, traffic not passing through the selected set of caches cannot be
displayed by means of log file analysis.

<H3>Frequent toplevel domains</H3>

For all selected hosts, the sums for a fixed set of top level domains is
displayed. Please note that some DFN caches are caching *.com where others
are caching !.com. A more advanced user interface might allow the user to
select interesting domains.

<H3>Frequent MIME types</H3>

For all selected hosts, the sums for a fixed set of MIME types is
displayed. A more advanced user interface might allow the user to select
interesting domains.

<H3>Top N toplevel domains</H3>

This is similar to the selection of toplevel domain, though this time,
a query determines the chart, and only the top of the chart will be
shown.

<H3>Top N AS destinations</H3>

The AS destinations shows, if you enabled seafood to do AS resolution,
the top of the chart destination autonomous systems for DIRECT connections.

<H3>Top N (TCP|UDP) (HIT)? clients</H3>

These four selection options will show the top of the client charts,
for UDP, UDP HITs, TCP, and TCP HITs.

<H3>Hierarchy performance</H3>

The performance data is evaluated for parents, siblings and direct,
and shown as combination of the selected caches.

</BODY>
</HTML>
EOF

